#include <Arduino.h>
#include "WiFi.h"
#include "WiFiMulti.h"
#include "src/InfluxDB-Client-for-Arduino/src/InfluxDbClient.h"
#include "src/EmonLib/EmonLib.h"
#include "config.h"

#define Apin 32
#define Vpin 33

EnergyMonitor emon1;             // Create an instance
WiFiMulti WiFiMulti;
InfluxDBClient client(INFLUXDB_URL, INFLUXDB_DB_NAME);

// Data point
Point sensor("solar_status");

void ConnectToWiFiMulti() {
  WiFi.mode(WIFI_STA);
  WiFiMulti.addAP(SSID, WiFiPassword);
  Serial.println("Connecting Wifi...");
  while (WiFiMulti.run() != WL_CONNECTED) {
    delay(100);
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void ConnectToInflux() {
  // Add constant tags - only once
  sensor.addTag("Phase", "Solar 1");

  // Check server connection
  if (client.validateConnection()) {
    Serial.print("Connected to InfluxDB: ");
    Serial.println(client.getServerUrl());
  } else {
    Serial.print("InfluxDB connection failed: ");
    Serial.println(client.getLastErrorMessage());
  }
}

void setup() {
  Serial.begin(115200);
  ConnectToWiFiMulti();
  ConnectToInflux();
  analogReadResolution(12);
  pinMode(Apin, INPUT);
  Serial.println(ADC_COUNTS);
  emon1.voltage(Vpin, 406, 1.7);  // Voltage: input pin, calibration, phase_shift
  emon1.current(Apin, 66);       // Current: input pin, calibration.
}

void loop() {
  Serial.println(analogRead(Vpin));
  emon1.calcVI(20, 2000);        // Calculate all. No.of half wavelengths (crossings), time-out
  Serial.print(emon1.Vrms);
  Serial.print(" Volts | ");
  Serial.print(emon1.Irms);
  Serial.print(" Amps | ");
  Serial.print(emon1.Vrms*emon1.Irms);
  Serial.println(" Watts");
  Serial.println(emon1.calcIrms(1480));
  //emon1.serialprint();           // Print out all variables (realpower, apparent power, Vrms, Irms, power factor)
  delay(5000);

  //influx to be implemented...
  Point sensor("wifi_status");
}
